/**
  \file EXTpad.h 
  Copyright (C) 2018 Xoán Sampaíño
  Based on Escornabot <https://github.com/escornabot>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef _EXT_PAD_H
#define _EXT_PAD_H

#include <Arduino.h>

const int BUTTON_PRESSED = 50;
const int BUTTON_PRESSED_LONG = 1000;

enum Button : byte {
  BUTTON_NONE = 0,
  BUTTON_FORWARD,
  BUTTON_FORWARD_LONG,
  BUTTON_BACK,
  BUTTON_BACK_LONG,
  BUTTON_LEFT,
  BUTTON_LEFT_LONG,
  BUTTON_RIGHT,
  BUTTON_RIGHT_LONG,
  BUTTON_GO,
  BUTTON_GO_LONG
};

class EXTpad {
  /**
    \class EXTpad EXTpad.h <EXTpad.h>
  */
  public:
    /**
      Constructor
      
      \param button_pin Analog pin
      \param none_value Analog value when BUTTON_NONE is pressed
      \param forward_value Analog value when BUTTON_FORWARD is pressed
      \param back_value Analog value when BUTTON_BACK is pressed
      \param left_value Analog value when BUTTON_LEFT is pressed
      \param right_value Analog value when BUTTON_RIGHT is pressed
      \param go_value Analog value when BUTTON_GO is pressed
    */
    EXTpad(byte button_pin = A0,
           int none_value = 0, int forward_value = 672, int back_value = 335,
           int left_value = 501, int right_value = 168, int go_value = 846);
    
    /**
    */
    void begin();
    
    /**
    */
    byte scan();
    
  private:
    byte _button_pin;
    int _value[6];
    byte _last_button = BUTTON_NONE;
    byte _pressed_button = BUTTON_NONE;
    bool _pressed = false;
    unsigned long _current_millis = 0;
};

#endif // _EXT_PAD_H

// EOF
